<?php

declare(strict_types=1);

namespace Dedi\SyliusBlogPlugin\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use JMS\Serializer\Annotation as Serializer;
use Sylius\Component\Core\Model\ChannelInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity()
 * @ORM\Table(name="dedi_blog_category")
 */
class Category extends AbstractReferable implements CategoryInterface
{
    /**
     * @ORM\Column(type="integer", nullable=false)
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     *
     * @Serializer\Expose()
     * @Serializer\Groups({"default", "Autocomplete"})
     */
    protected ?int $id;

    /**
     * @ORM\Column(type="boolean", nullable=false, name="category_enabled")
     */
    protected bool $enabled = true;

    /**
     * @ORM\Column(type="integer", nullable=false, name="category_position")
     * @Assert\PositiveOrZero(groups={"default", "sylius"})
     */
    protected ?int $position = 0;

    /**
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusBlogPlugin\Entity\Article", mappedBy="categories")
     */
    protected Collection $articles;

    /**
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusBlogPlugin\Entity\Category", mappedBy="subCategories")
     */
    protected Collection $parents;

    /**
     * @ORM\ManyToMany(targetEntity="Sylius\Component\Core\Model\ChannelInterface")
     * @ORM\JoinTable(name="dedi_blog_categories_channels",
     *     joinColumns={@ORM\JoinColumn(name="category_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="channel_id", referencedColumnName="id")}
     * )
     */
    protected Collection $channels;

    /**
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusBlogPlugin\Entity\Category", inversedBy="parents", cascade={"persist"})
     * @ORM\JoinTable(name="dedi_blog_categories_subcategories",
     *     joinColumns={@ORM\JoinColumn(name="category_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="subcategory_id", referencedColumnName="id")}
     * )
     */
    protected Collection $subCategories;

    public function __construct()
    {
        parent::__construct();

        $this->articles = new ArrayCollection();
        $this->parents = new ArrayCollection();
        $this->subCategories = new ArrayCollection();
        $this->channels = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getPosition(): int
    {
        return $this->position;
    }

    public function setPosition(int $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function getSlug(): string
    {
        return (string) $this->getTranslation()->getSlug();
    }

    public function setSlug(string $slug): self
    {
        $this->getTranslation()->setSlug($slug);

        return $this;
    }

    /**
     * @Serializer\VirtualProperty(name="name")
     * @Serializer\Groups({"default", "Autocomplete"})
     */
    public function getName(): string
    {
        return (string) $this->getTranslation()->getName();
    }

    public function setName(string $name): self
    {
        $this->getTranslation()->setName($name);

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->getTranslation()->getDescription();
    }

    public function setDescription(?string $description): self
    {
        $this->getTranslation()->setDescription($description);

        return $this;
    }

    public function getChannels(): Collection
    {
        return $this->channels;
    }

    public function setChannels(Collection $channels): self
    {
        $this->channels = $channels;

        return $this;
    }

    public function addChannel(ChannelInterface $channel): bool
    {
        if (!$this->channels->contains($channel)) {
            return $this->channels->add($channel);
        }

        return false;
    }

    public function removeChannel(ChannelInterface $channel): bool
    {
        return $this->channels->removeElement($channel);
    }

    public function getArticles(): \Traversable
    {
        return $this->articles;
    }

    public function hasArticles(): bool
    {
        return !$this->articles->isEmpty();
    }

    public function getSubCategories(): Collection
    {
        return $this->subCategories;
    }

    public function setSubCategories(Collection $subCategories): self
    {
        $this->subCategories = $subCategories;

        return $this;
    }

    public function hasSubCategories(): bool
    {
        return !$this->subCategories->isEmpty();
    }

    public function addSubCategory(CategoryInterface $category): bool
    {
        if (!$this->subCategories->contains($category)) {
            return $this->subCategories->add($category);
        }

        return false;
    }

    public function removeSubCategory(CategoryInterface $category): bool
    {
        return $this->subCategories->removeElement($category);
    }

    public function getParents(): Collection
    {
        return $this->parents;
    }

    public function hasParents(): bool
    {
        return !$this->parents->isEmpty();
    }

    public function getMetadataTitle(): ?string
    {
        return $this->getTranslation()->getMetadataTitle() ?? $this->getName();
    }

    public function getMetadataDescription(): ?string
    {
        return $this->getTranslation()->getMetadataDescription() ??
            ($this->getDescription() ? substr($this->getDescription(), 0, 155) : null);
    }

    public function getTranslation(?string $locale = null): CategoryTranslation
    {
        /** @var CategoryTranslation $translation */
        $translation = parent::getTranslation($locale);

        return $translation;
    }

    protected function createTranslation(): CategoryTranslation
    {
        return new CategoryTranslation();
    }

    public function __toString(): string
    {
        return $this->getName();
    }
}
